import logging
import re
import sys
import unittest
from pathlib import Path

from chromaformatter import ChromaFormatter, Colors


class UtilTest(unittest.TestCase):
    def __init__(self, *args) -> None:
        self.log_path = "log/test.log"
        self.log = logging.getLogger()
        log_format = (
            f"{Colors.Fore.GREEN}[%(asctime)-0s]"
            f"{Colors.LEVEL_COLOR}[%(levelname)-0s]"
            f"{Colors.Fore.MAGENTA}[%(filename)-0s:"
            f"%(lineno)-0d]"
            f"{Colors.LEVEL_COLOR}: %(message)s"
        )
        self.formatter = ChromaFormatter(
            log_format, Colors.Fore.WHITE, Colors.LEVEL_COLOR
        )
        file_handler = logging.FileHandler(self.log_path, mode="w")
        file_handler.setFormatter(self.formatter)
        stream_handler = logging.StreamHandler(stream=sys.stdout)
        stream_handler.setFormatter(self.formatter)
        self.log.addHandler(stream_handler)
        self.log.addHandler(file_handler)
        self.log.setLevel(logging.DEBUG)
        super(UtilTest, self).__init__(*args)

    def test_formatter_style(self) -> None:
        # noinspection PyProtectedMember
        self.assertEqual(
            self.formatter._style._fmt.encode(),
            f"{Colors.Fore.GREEN}[%(asctime)-0s]$LEVEL[%(levelname)-0s]"
            f"{Colors.Fore.MAGENTA}[%(filename)-0s:%(lineno)-0d]"
            f"{Colors.LEVEL_COLOR}: %(message)s"
            f"{Colors.Style.RESET_ALL}".encode(),
        )

    def test_log_color(self) -> None:
        # Test that a log message is colored as it should be.
        self.log.info(
            "It was not until [%s] that light was identified as the source of"
            " the color sensation.",
            "Newton",
        )
        file_name = Path(__file__).name
        with open(self.log_path) as f:
            log_file_text = f.read()
            # Replace timestamp time with the text 'timestamp' to check
            # against.
            log_file_text = re.sub(
                r"\[\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2},\d{3}]",
                "[timestamp]",
                log_file_text,
            )
            # Replace the line number with the text 'lineno' to check
            # against.
            log_file_text = re.sub(
                fr"\[({file_name}):\d+]", r"[\1:lineno]", log_file_text
            )
            self.assertEqual(
                log_file_text.encode(),
                f"{Colors.Fore.GREEN}[timestamp]{Colors.Fore.CYAN}[INFO]"
                f"{Colors.Fore.MAGENTA}[{file_name}:lineno]"
                f"{Colors.Fore.CYAN}: It was not until"
                f" [{Colors.Fore.WHITE}Newton{Colors.Fore.CYAN}]"
                f" that light was identified as the source"
                f" of the color sensation.{Colors.Style.RESET_ALL}\n".encode(),
            )

    def test_something(self) -> None:
        # Test that formatter is updated based on log.
        # noinspection PyProtectedMember
        self.assertEqual(
            self.formatter._style._fmt.encode(),
            f"{Colors.Fore.GREEN}[%(asctime)-0s]"
            f"{Colors.Fore.CYAN}[%(levelname)-0s]"
            f"{Colors.Fore.MAGENTA}[%(filename)-0s:%(lineno)-0d]"
            f"{Colors.Fore.CYAN}: %(message)s"
            f"{Colors.Style.RESET_ALL}".encode(),
        )


if __name__ == "__main__":
    unittest.main()



import re
from logging import Formatter, DEBUG, INFO, WARNING, ERROR, CRITICAL, LogRecord, NOTSET
from typing import Dict, Optional

import colorama

__all__ = (
    "ChromaFormatter",
    "Colors",
)


class Colors:
    Fore = colorama.Fore
    Back = colorama.Back
    Style = colorama.Style
    LEVEL_COLOR = "$LEVEL"


# noinspection PyProtectedMember
class ChromaFormatter(Formatter):
    """Extended logging.Formatter to add colors and styles."""

    def __init__(
        self,
        fmt: str,
        arg_start_color: Optional[str] = None,
        arg_end_color: Optional[str] = None,
    ) -> None:
        """Set ChromaFormatter properties.

        :param fmt: The format string to determine how logs will appear.
        :param arg_start_color: Color of formatted arguments.
        :param arg_end_color: Color after formatted arguments.
        """
        fmt = f"{fmt}{Colors.Style.RESET_ALL}"
        super().__init__(fmt)
        self._original_style_fmt = self._style._fmt
        self.arg_start_color = arg_start_color
        self.arg_end_color = arg_end_color

        self.color_map: Dict[int, str] = {
            NOTSET: Colors.Fore.LIGHTBLUE_EX,
            DEBUG: Colors.Fore.BLUE,
            INFO: Colors.Fore.CYAN,
            WARNING: Colors.Fore.YELLOW,
            ERROR: Colors.Fore.LIGHTRED_EX,
            CRITICAL: Colors.Fore.RED,
        }

    def format(self, record: LogRecord) -> str:
        """Format and add colors to a log record.

        :param record: LogRecord to format and color.
        :return: The complete log record formatted and colored.
        """
        msg = str(record.msg)
        if record.levelno not in self.color_map:
            return super(ChromaFormatter, self).format(record)
        self._style._fmt = self._original_style_fmt
        level_color = self.color_map[record.levelno]
        # Color the record msg.
        if record.args and self.arg_start_color and self.arg_end_color:
            record.msg = re.sub(
                r"(?<!%)%([-0.\d]*)([sd])",
                fr"{self.arg_start_color}%\1\2{self.arg_end_color}",
                record.msg,
            ).replace("$LEVEL", level_color)
        self._style._fmt = self._style._fmt.replace(Colors.LEVEL_COLOR, level_color)
        s = super(ChromaFormatter, self).format(record)
        record.msg = msg
        return s
